#!/usr/bin/env python3
import ctypes
from ctypes import *
from enum import Enum
from pydbus import SystemBus
import syslog

KLOG_ERROR   = 3
KLOG_WARNING = 4
KLOG_NOTICE  = 5
KLOG_INFO    = 6
KLOG_DEBUG   = 7

KLOG_TAG_CALL = "Call"
KLOG_TAG_EXIT = "Exit"

class KdkLogType(Enum):
    KDK_SEC_LOG_TYPE_COMM = 0
    KDK_SEC_LOG_TYPE_DBUS = 1

class KdkLogModule(Enum):
    KDK_MODULE_ACCOUNT = 0                      #账户安全
    KDK_MODULE_APPLICATION = 1                  #应用安全
    KDK_MODULE_DATA = 2                         #数据安全
    KDK_MODULE_DESKTOP = 3                      #桌面管控
    KDK_MODULE_DEVICE = 4                       #设备安全
    KDK_MODULE_FILE = 5                         #文件安全（防篡改）
    KDK_MODULE_NETWORK = 6                      #网络安全
    KDK_MODULE_PROCESS = 7                      #进程安全
    KDK_MODULE_USERAUTH = 8                     #用户身份识别
    KDK_MODUlE_LOG = 9                          #安全SDK日志
    KDK_MODULE_SECDAEMON = 10                   #安全SDK初始化
    KDK_MODULE_APPLICATION_SYSSECRISKBOX = 11   #应用安全system风险提示服务
    KDK_MODULE_MAX = 12


TIME_MAX_SIZE = 100
MESG_MAX_SIZE = 1024
FUNC_MAX_SIZE = 512
PROC_MAX_SIZE = 512
DBUS_MAX_SIZE = 256
class KdkLogNodeStruct(Structure):
    _fields_ = [
        ('level', c_int),
        ('module', c_int),
        ('pid', c_int),
        ('uid', c_int),
        ('path', c_char * PROC_MAX_SIZE),
        ('func', c_char * FUNC_MAX_SIZE),
        ('time', c_char * TIME_MAX_SIZE),
        ('mesg', c_char * MESG_MAX_SIZE),
        ('type', c_int),
        ('dbus_name', c_char * DBUS_MAX_SIZE),
        ('dbus_path', c_char * DBUS_MAX_SIZE),
        ('dbus_interface', c_char * DBUS_MAX_SIZE)
    ]

class KdkSecLog:
    def __init__(self):
        try:
            self.liblogsec = CDLL("libkylogsec.so")
            self.dbus = SystemBus().get("org.freedesktop.DBus", "/org/freedesktop/DBus")
        except Exception:
            exit(-1)

    def kdklog(self, level, module, pid, uid, type, dbus_name, dbus_path, dbus_interface, func, message):
        try:
            klog_node = KdkLogNodeStruct()
            self.liblogsec.kdk_log_init(byref(klog_node), c_int(level), c_int(module), c_int(pid), c_int(uid), c_int(type), 
                                        create_string_buffer(dbus_name.encode()), create_string_buffer(dbus_path.encode()), 
                                        create_string_buffer(dbus_interface.encode()), create_string_buffer(func.encode()))
            self.liblogsec.kdk_sec_log_node(klog_node, create_string_buffer(message.encode()))
        except Exception as e:
            syslog.syslog(syslog.LOG_DEBUG, "kysdk log error : {}".format(e))
            pass

    def kdklog_dbus(self, sender, module, dbus_name, dbus_path, dbus_interface, func, message=None):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            self.kdklog(KLOG_INFO, module, pid, uid, KdkLogType.KDK_SEC_LOG_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, message)

    def kdklog_dbus_call(self, sender, module, dbus_name, dbus_path, dbus_interface, func):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            self.kdklog(KLOG_INFO, module, pid, uid, KdkLogType.KDK_SEC_LOG_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, KLOG_TAG_CALL)

    def kdklog_dbus_exit(self, sender, module, dbus_name, dbus_path, dbus_interface, func):
        if sender != None:
            pid = self.dbus.GetConnectionUnixProcessID(sender)
            uid = self.dbus.GetConnectionUnixUser(sender)
            self.kdklog(KLOG_INFO, module, pid, uid, KdkLogType.KDK_SEC_LOG_TYPE_DBUS.value, dbus_name, dbus_path, dbus_interface, func, KLOG_TAG_EXIT)
